#! /usr/bin/perl
#Name: lander-client-commands.perl
#Project: Lander control
#Homepage: www.HomoFaciens.de
#Author Norbert Heinz
#Version: 1.0
#Creation date: 03.01.2012
#Copyright: (c) 2011 by Norbert Heinz
#This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program if not, see http://www.gnu.org/licenses/.
#
#Server script for rover-client-commands.perl. Both scripts are used to send and receive commands via WLAN.

use strict;
use IO::Socket;

use FindBin qw($Bin);

#$|=1 ;

$SIG{'QUIT'} = 'interrupt';
$SIG{'INT' } = 'interrupt';
$SIG{'HUP' } = 'interrupt';
$SIG{'TRAP'} = 'interrupt';
$SIG{'ABRT'} = 'interrupt';
$SIG{'STOP'} = 'interrupt';

my $socket;
my $received_data;
my $send_data;
my $client_socket;
my $peer_address;
my $peer_port;
my $server_port = "1099"; #Must be the same at 'rover-client-commands.perl'
my $send_data;
my $aut_ok = 0;
my $shell_command = "";
my @file_info;
my ($seconds, $minutes, $hours, $day_of_month, $month, $year, $weekday, $day_of_year, $summertime) = localtime(time);
my $CTIME_String = localtime(time);
my $passwd_line = "";
my $char_temp = "";
my $command_line = "";


sub interrupt {#catch interrupt signals
  my($signal)=@_;
  &LogFile("Caught Interrupt\: $signal");
  print "Exiting program\n";
  exit(1);
}

print "\n\n";

chdir("$Bin");

  &LogFile ("Starting client commands");

  $socket = new IO::Socket::INET (
                                    LocalHost => '192.168.1.xxx',     #Type the IP adress or the name of the Lander's hostcomputer here
                                    LocalPort => $server_port,
                                    Proto => 'tcp',
                                    Listen => 1,    #number of clients to accept
                                    Reuse => 1
                                );

  die "Could not open socket" unless $socket;


while (111 == 111){

  print "Waiting for client on port " . $server_port . "\n";

  $client_socket = "";
  $client_socket = $socket->accept();

  $peer_address = $client_socket->peerhost();
  $peer_port = $client_socket->peerport();

  &LogFile ("Incomming connection from address '" . $peer_address . "', port '" . $peer_port . "'");

  $aut_ok=1;

  &ReceiveData(33);

  #Password must be identically at Rover script (rover-client-commands.perl).
  $passwd_line = substr("TypeYourPasswordHere" . "                                ", 0, 32);

  if(substr($received_data, 0, 32) eq $passwd_line){
     $aut_ok = 1;
     print "Authentication OK\n"
  }
  else{
    $aut_ok = 0;
    &LogFile ("Authentication failed! '" . $received_data . "' from address '" . $peer_address . "', port'" . $peer_port . "'");
  }

  if($aut_ok == 1){
    &ReceiveData(20);
    print"Received >" . $received_data . "<\n";
    $char_temp = time();
    &LogFile("Received>$received_data<");
    if(substr($received_data, 0, 15) eq "LanderCommand=1"){
      $command_line = "killall -s SIGTERM chromium-browser";
      print "command_line=>" . $command_line . "<\n";
      `$command_line`;
      $command_line = "/usr/bin/chromium-browser --kiosk /home/knoppix/lander/lander.htm?" . $char_temp . "&";
      print "command_line=>" . $command_line . "<\n";
      system($command_line);
      $command_line = "";
    }
    if(substr($received_data, 0, 15) eq "LanderCommand=2"){
      $command_line = "killall -s SIGTERM chromium-browser";
      print "command_line=>" . $command_line . "<\n";
      `$command_line`;
      $command_line = "/usr/bin/chromium-browser --kiosk /home/knoppix/lander/clock.htm?" . $char_temp . "&";
      print "command_line=>" . $command_line . "<\n";
      system($command_line);
      $command_line = "";
    }
    if(substr($received_data, 0, 15) eq "LanderCommand=3"){
      $command_line = "killall -s SIGTERM chromium-browser";
      print "command_line=>" . $command_line . "<\n";
      `$command_line`;
      $command_line = "/usr/bin/chromium-browser --kiosk www.en.wikipedia.com?" . $char_temp . "&";
      print "command_line=>" . $command_line . "<\n";
      system($command_line);
      $command_line = "";
    }
    if(substr($received_data, 0, 15) eq "LanderCommand=4"){
      $command_line = "killall -s SIGTERM chromium-browser";
      print "command_line=>" . $command_line . "<\n";
      `$command_line`;
      $command_line = "/usr/bin/chromium-browser --kiosk /home/knoppix/lander/rover.htm?" . $char_temp . "&";
      print "command_line=>" . $command_line . "<\n";
      system($command_line);
      $command_line = "";
    }
  }# if($aut_ok == 1)
  close $client_socket;
  &LogFile("Client socked closed.");
}#while(111 == 111);

&FileLog("Server Script aborted?!?");


#++++++++++++++++++++ ReceiveData +++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub ReceiveData{
  if($aut_ok == 1){
    $received_data="";
    $client_socket->recv($received_data, $_[0], MSG_WAITALL);
    if(length($received_data) + 1 < $_[0]){
      &LogFile ("Receipt of data interrupted. received_data='" . $received_data . "' _[0]=" . $_[0] . ", length received_data=" . length($received_data));
      $received_data="";
      $aut_ok = 0;
      #die "Receipt of data interrupted.\n";
    }
  }
}

#++++++++++++++++++++ SendData +++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub SendData{
  if($aut_ok == 1){
    print "send_data='" . $_[0] . "'\n";
    eval {$client_socket->send($_[0], $_[1]);};
    if ($@) {
      &LogFile ("Sending of data interrupted. send_data='" . $send_data . "'");
      $aut_ok = 0;
    }
  }
}


#++++++++++++++++++++++ LogFile +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
sub LogFile{

  @file_info=stat("lander-client-commands.log");

  ($seconds, $minutes, $hours, $day_of_month, $month, $year, $weekday, $day_of_year, $summertime) = localtime(time());

  if ($hours < 10){
    $hours = "0" . $hours;
  }
  if ($minutes < 10){
    $minutes = "0" . $minutes;
  }
  if ($seconds < 10){
    $seconds = "0" . $seconds;
  }
  $month += 1;
  $year += 1900;

  if ($month < 10){
    $month = "0" . $month;
  }
  if ($day_of_month < 10){
    $day_of_month = "0" . $day_of_month;
  }

  $char_temp=substr($day_of_month . "." . $month . "." . $year . "-" . $hours . ":" . $minutes . ":" . $seconds . "                    ", 0, 20);

  if ($file_info[7] > 1000000){
    `mv lander-client-commands.log lander-client-commands01.log`
  }

  open(LOGFILE, ">> lander-client-commands.log") || die "Can not open file 'lander-client-commands.log'!\n";
    print LOGFILE $char_temp . $_[0] . "\n";
  close(LOGFILE);

}